/*-
 * Copyright (c) 2002  Granch Ltd. 	All rigts reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	@(#)readconfig.c    		0.91.1 (Granch Ltd.) 	30/10/02
 */
/*------------------------------------------------------------------------

            Standard functions library

 	    UNIX FreeBSD 4.6 version

            Read and drop spaces and commented lines from config file
            Also drops horizontal and vertical tab characters

	    Call : int readconfig(char commchar, char *inbuf,
					char *outbuf, int buffersize);

            Return : buffersize - file read and parsed.
        	     -1 - Some errors were detected

--------------------------------------------------------------------------*/
#include "make.h"

int readconfig(char commchar,char *inbuf,char *outbuf,int buffersize)
{
  int i,j,k = 1;
  char *lin,*lout;

// Line-by-line config buffer processing.

  for (i = j = 0,lin = inbuf,lout = outbuf;i < buffersize;)
   {

// Skip a empty line (only one \n per line)
     if ((*lin == '\n') && (*(lin + 1) == '\n'))
       {
         lin++;
         i++;
         continue;
       }

// Skip commented line (with a commchar as first char)
     if (*lin == commchar)
       {
         if ((j = ((char *) memchr(lin,'\n',buffersize)) - lin + 1) <= NULL)
           return ERR;
         lin += j;
         i += j;
         continue;
       }
      else

// We should keep spaces inside any line, quoted in double-quote char (")
// but quotes itself will be deleted

        {
          switch(*lin)
           {
             case HZTAB:
             case VTTAB:
             case SPACE: if (k) lin++;
                           else
                             *(lout++) = *(lin++);
             		 break;

             case QUOTA: lin++;
                         if (k) k = 0;
                           else k = 1;
                         break;

             default: *(lout++) = *(lin++);
           }
          i++;
        }
   }

// Check to '\n' at first char (may appear, when before first parameter were empty
// lines)

  if (*outbuf == '\n')
    {
      memmove(outbuf, outbuf + 1, lout - outbuf - 1);
      *(lout - 1) = '\n';
      return(lout - outbuf - 1);
    }
   else
     return(lout - outbuf);
}
